﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("HoldoutLeafPruning", "Postpruning via a holdout set. Pruning is done using the model type of the leaf models")]
  public class HoldoutLeafPruning : PruningBase {
    private const string HoldoutSizeParameterName = "HoldoutSize";
    public IFixedValueParameter<PercentValue> HoldoutSizeParameter {
      get { return Parameters[HoldoutSizeParameterName] as IFixedValueParameter<PercentValue>; }
    }
    public double HoldoutSize {
      get { return HoldoutSizeParameter.Value.Value; }
    }

    #region Constructors & Cloning
    [StorableConstructor]
    protected HoldoutLeafPruning(bool deserializing) : base(deserializing) { }
    protected HoldoutLeafPruning(HoldoutLeafPruning original, Cloner cloner) : base(original, cloner) { }
    public HoldoutLeafPruning() : base() {
      Parameters.Add(new FixedValueParameter<PercentValue>(HoldoutSizeParameterName, new PercentValue(0.2)));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new HoldoutLeafPruning(this, cloner);
    }
    #endregion

    #region PruningType
    public override ILeafType<IRegressionModel> ModelType(ILeafType<IRegressionModel> leafType) {
      return leafType;
    }

    public override void GenerateHoldOutSet(IReadOnlyList<int> allrows, IRandom random, out IReadOnlyList<int> training, out IReadOnlyList<int> holdout) {
      training = new List<int>();
      holdout = new List<int>();
      foreach (var row in allrows)
        ((List<int>) (random.NextDouble() < HoldoutSize ? holdout : training)).Add(row);
    }
    #endregion
  }
}